'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var core = require('@angular/core');

function autoChangeDetection(component, inputs, props) {
    const keys = Object.keys(inputs);
    keys.forEach(key => {
        props.set(key, component[key]);
        Object.defineProperty(component, key, {
            get() {
                return props.get(key);
            },
            set(value) {
                props.set(key, value);
                core.ɵdetectChanges(component);
            }
        });
    });
    return component;
}
function initProps(target, inputs, component) {
    const keys = Object.keys(inputs);
    keys.forEach(key => {
        Object.defineProperty(target, key, {
            get() {
                return component[key];
            },
            set(value) {
                component[key] = value;
            }
        });
    });
}
function initEvents(target, outputs, component) {
    const outputEvents = Object.keys(outputs);
    outputEvents.forEach(outputEvent => {
        component[outputs[outputEvent]].subscribe(info => {
            target.dispatchEvent(new CustomEvent(outputEvent, { detail: info }));
        });
    });
}
function getDirectiveDef(t) {
    if (t['ngDirectiveDef']) {
        return t['ngDirectiveDef'];
    }
    if (t['ngComponentDef']) {
        return t['ngComponentDef'];
    }
    throw new Error('No Angular definition found for ' + t.name);
}
function getPipeDef(t) {
    if (t['ngPipeDef']) {
        return t['ngPipeDef'];
    }
    throw new Error('No Angular definition found for ' + t.name);
}
function updateComponentDef(componentType) {
    const def = componentType['ngComponentDef'];
    const directives = (componentType['deps'] && componentType['deps']['directives'])
        ? componentType['deps']['directives'] : null;
    const pipes = (componentType['deps'] && componentType['deps']['pipes'])
        ? componentType['deps']['pipes'] : null;
    def.directiveDefs = (directives)
        ? directives.map(directive => getDirectiveDef(directive))
        : null;
    def.pipeDefs = (pipes)
        ? pipes.map(pipe => getPipeDef(pipe))
        : null;
    return def;
}

function renderNgComponent(componentType, opts) {
    const props = new Map();
    const def = updateComponentDef(componentType);
    const component = core.ɵrenderComponent(componentType, opts);
    return autoChangeDetection(component, def.inputs, props);
}
function createCustomElement(componentType, injector) {
    const { inputs, styles, encapsulation, outputs } = componentType['ngComponentDef'];
    return class CustomElement extends HTMLElement {
        constructor() {
            super();
            this.rootElement = (encapsulation === core.ViewEncapsulation.ShadowDom)
                ? this.attachShadow({ mode: "open" }) : this;
            this.component = renderNgComponent(componentType, {
                host: this.rootElement,
                ...(injector ? { injector } : {})
            });
            initProps(this, inputs, this.component);
            initEvents(this, outputs, this.component);
        }
        static get observedAttributes() {
            return [...Object.keys(inputs)];
        }
        attributeChangedCallback(name, oldValue, newValue) {
            const attributes = Object.keys(inputs);
            if (attributes.includes(name)) {
                this[name] = newValue;
            }
        }
        connectedCallback() {
            const cssStyles = styles.join('\n');
            if (cssStyles) {
                const style = document.createElement('style');
                style.textContent = cssStyles;
                this.rootElement.prepend(style);
            }
        }
    };
}
function renderCustomElement(componentType, injector) {
    customElements.define(componentType['ngComponentDef'].selectors[0][0], createCustomElement(componentType, injector));
}

function NgxElement(metadata) {
    return (component) => {
        component.deps = {
            directives: metadata.directives,
            pipes: metadata.pipes
        };
        core.ɵcompileComponent(component, metadata);
        return component;
    };
}

exports.NgxElement = NgxElement;
exports.createCustomElement = createCustomElement;
exports.renderCustomElement = renderCustomElement;
exports.renderNgComponent = renderNgComponent;
